#!/usr/bin/perl
#
# areasbbsadm.pl - v0.2 
# Administration of the areasbbs or fareasbbs tables
# - Createing an areasbbs table in an SQL (Mysql) based database.
# - Createing an fareasbbs table in an SQL (Mysql) based database.
# - Adding descriptions to either table from a file (not yet implemented)
#  Copyright (c) 2001-2004,2010 Robert James Clay.  All Rights Reserved.
#  This is free software;  you can redistribute it and/or
#  modify it under the same terms as Perl itself.

use warnings;
use strict;
use Getopt::Std;
use vars qw/ $opt_h $opt_a $opt_d $opt_f $opt_l $opt_v $opt_x /;

use FTN::Log qw(&logging);

my ($db_handle, $sql_statement, $Debug, $description_file, $log_file);

#my $log_id = "abadm";

getopts('d:l:afhvx');

if($opt_h) {
    print "Usage: areasbbsadm.pl [-f] [-l STDOUT|log_file] [-v] [-x]...\n";
    print "  -a  create areasbbs table, or...\n";
    print "  -f  create file areasbbs table...\n";
    print "  -d  description file pathname .\n";
    print "  -l  stdout or logfile pathname .\n";
    print "  -v  Verbose \n";
    print "  -x  Set debug...\n\n";
    exit;
}

if ($opt_l) {
    $log_file=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
    $log_file="stdout";		# defaults to standard out.
}

if ($opt_d) {
    $description_file=$opt_d;  # this needs to be the filename & path
    undef $opt_d;
} else {
    $description_file="echodesc.txt";		# defaults to echodesc.txt.
}

if ($opt_v) {
    print "logfile is $log_file.\n";
    &log("Verbose flag is set ...");
}
	
if ($opt_x) {
    $Debug = $opt_x;	# set debug flag
    undef $opt_x;
    &log("Debug flag is set ...");
} else {
    $Debug = 0;
}



if ($opt_a) {
    &createareastable();

} elsif ($opt_f) {
    &createareastable();

} elsif ($description_file) {
    &UpdateAreaDescriptions();
}

#  
exit();

############################################
# create areasbbs or fareasbbs table
############################################
sub createareastable {

# connect to database
    &openftndb();	# open message database

## drop the old echo table, if it exists.
## uncomment this if needed.
#    if ($opt_f) {
#        $sql_statement = "DROP TABLE fareasbbs";
#    } else {
#	$sql_statement = "DROP TABLE areasbbs";
#    }
#
#    print " $sql_statement ";
#    $db_handle->do( "$sql_statement " ) or  die $DBI::errstr;
#        or $drop_ok = "0"; 

# build Create Table sql statement
    if ($opt_f) {
        $sql_statement = "CREATE TABLE fareasbbs( ";
    } else {
        $sql_statement = "CREATE TABLE areasbbs( ";
    }

    $sql_statement .= "id              INT NOT NULL AUTO_INCREMENT PRIMARY KEY, ";

# areaname     name as distributed
    $sql_statement .= "areaname        VARCHAR(32) DEFAULT '' NOT NULL, ";

# bbsname      name of table in MySQL database
    $sql_statement .= "bbsname         VARCHAR(32) DEFAULT '' NOT NULL, ";

#  description    
    $sql_statement .= "description     VARCHAR(128) DEFAULT '' NOT NULL, ";

# prinode
    $sql_statement .= "prinode         VARCHAR(72) DEFAULT ' ' NOT NULL, ";

# prinode
    $sql_statement .= "uplink          VARCHAR(72) DEFAULT ' ' NOT NULL, ";

#  domain       
    $sql_statement .= "domain          VARCHAR(8) DEFAULT 'fidonet' NOT NULL, ";

    if ($opt_f) {	# do fareasbbs table specific fields
#  file system path where files for this file echo are put
        $sql_statement .= "dirpath     VARCHAR(128) DEFAULT '' NOT NULL, ";

    } else {	# do areasbbs table specific fields
#  maxmsgs    Maximum number of msgs to keep                   
        $sql_statement .= "maxmsgs     MEDIUMINT DEFAULT '100' NOT NULL, ";

#  maxdays    Maximum number of days to keep msgs
        $sql_statement .= "maxdays     TINYINT DEFAULT '28' NOT NULL, ";
#  no real boolean type?
#  type  L=local, N=netmail, C=echo message area	
        $sql_statement .= "type        CHAR(1) DEFAULT 'L' NOT NULL, ";
    }	

#   What to do for the index?
    if ($opt_f) {
        $sql_statement .= "INDEX fareasbbsidx (id, areaname) ";
    } else {
        $sql_statement .= "INDEX areasbbsidx (id, areaname) ";
    }

    $sql_statement .= ") ";

#print " $sql_statement ";
    $db_handle->do( "$sql_statement " ) or die $DBI::errstr;

# disconnect from database
    &closeftndb();

}

###################################################
# Update area descriptions in (f)areas bbs table
###################################################
sub UpdateAreaDescriptions {
# This subroutine adds conference (echo) descriptions from
# a file to the areasbbs table.  

    my ($distribution_name, $desc); 
    
    open (DESCFILE, $description_file)
        or die &log("Cannot open Echo Description file:  $description_file");
      
# connect to database
    &openftndb();	# open message database

#   this is the begining of the read file loop
    while(<DESCFILE>) {

        ($distribution_name,$desc) = split(' ', $_, 2);
      
	$distribution_name =~ tr/\"/ /;  # remove qoutes from name
	$distribution_name =~ tr/\'/ /;  # remove qoutes from name
	
        chomp($desc);	# remove CR
#	$desc =~ tr/\"/\'/;  # change double quotes to single

	if ($opt_v) {&log("updateing description for $distribution_name.")};

        $sql_statement = "UPDATE areasbbs ";

#        $sql_statement .= "SET description='$desc' ";

        $sql_statement .= "SET description= ";


        $sql_statement .= $db_handle->quote($desc);	#

        $sql_statement .= " WHERE areaname = '$distribution_name' ";

        if($opt_x) {

		print "SQL Statement:  $sql_statement \n";
        }

    $db_handle->do( "$sql_statement " );

#  this is the end of the read file loop
}

# disconnect from database
    &closeftndb();

    close DESCFILE;

}
############################################
# open FTN MySQL database for operations
############################################
sub openftndb {
# Open message database
#   Assumes that $db_name, $db_user, & $db_password
# have been set.  $db_user must already 
# have the priveledges to create a table.

    if ($opt_v) {&log("Opening FTN database")};
	
    my $db_name = 'ftndbtst';
    my $db_user = 'sysop';
    my $db_password = 'ftntstpw';
 
    use DBI;

    ( $db_handle = DBI->connect("dbi:mysql:dbname=$db_name", $db_user, $db_password) )
	or die &log($DBI::errstr);

}

############################################
# Close FTN 
############################################
sub closeftndb {
#
    if ($opt_v) {&log("Closing FTN database")};

    ( $db_handle->disconnect )
	or die &log($DBI::errstr);
	
}

#############################################
#  logging subrouting.  requires FTN::Log
#############################################
sub log {
#
    my(@text) = @_;
    my $log_id="abadm";

    &logging($log_file, $log_id, @text);

}
############################################
